local auraWhiteList = {
		["Power Word: Fortitude"] = true,
		["Prayer of Fortitude"] = true,
		["Shadow Protection"] = true,
		["Prayer of Shadow Protection"] = true,
		["Divine Spirit"] = true,
		["Prayer of Spirit"] = true,
		["Arcane Brilliance"] = true,
		["Arcane Intellect"] = true,
		["Dalaran Brilliance"] = true,
		["Gift of the Wild"] = true,
		["Mark of the Wild"] = true,
		["Soulstone Resurrection"] = true,
		["Heroic Presence"] = nil,
	}

local colors = setmetatable({
	power = setmetatable({
		['MANA'] = {0, 144/255, 1},
	}, {__index = oUF.colors.power}),
}, {__index = oUF.colors})

local function menu(self)
	if(self.unit:match('party')) then
		ToggleDropDownMenu(1, nil, _G['PartyMemberFrame'..self.id..'DropDown'], 'cursor')
	else
		FriendsDropDown.unit = self.unit
		FriendsDropDown.id = self.id
		FriendsDropDown.initialize = RaidFrameDropDown_Initialize
		ToggleDropDownMenu(1, nil, FriendsDropDown, 'cursor')
	end
end

oUF.Tags['[smarthp]'] = function(u)
	local min, max = UnitHealth(u), UnitHealthMax(u)
	return UnitIsDeadOrGhost(u) and oUF.Tags['[dead]'](u) or (min~=max) and format('|cffff8080%d|r|cff0090ff %.0f|r%%', min-max, min/max*100) or max
end

oUF.Tags['[smartpp]'] = function(u)
	local min, max = UnitPower(u), UnitPowerMax(u)
	return (UnitPowerType(u) == 0 and min > 0) and format('|cff%02x%02x%02x%.0f|r%%', 0, 144, 255, min/max*100)
end

oUF.Tags['[afk]'] = function(u) return UnitIsAFK(u) and '<AFK>' end
oUF.Tags['[assisticon]'] = function(u) local i = GetRaidTargetIndex(u..'target') return i and ICON_LIST[i]..'22|t' end
oUF.Tags['[assistname]'] = function(u) return UnitName(u..'target') end

oUF.TagEvents['[smarthp]'] = 'UNIT_HEALTH'
oUF.TagEvents['[smartpp]'] = 'UNIT_MANA UNIT_DISPLAYPOWER'
oUF.TagEvents['[afk]'] = 'PLAYER_FLAGS_CHANGED'
oUF.TagEvents['[assisticon]'] = 'UNIT_TARGET RAID_TARGET_UPDATE'
oUF.TagEvents['[assistname]'] = 'UNIT_TARGET UNIT_NAME_UPDATE'

--INSERT FRAME FOR ICON
local function createAura(self, button, icons)
	icons.showType = true
	button.cd:SetReverse()
	button.overlay:SetTexture([=[Interface\AddOns\oUF_Perfect2\border]=])
	button.overlay:SetTexCoord(0, 1, 0, 1)
end


local function customFilter(icons, unit, icon, name, rank, texture, count, dtype, duration, timeLeft, caster)
	icon.duration, icon.timeLeft = duration, timeLeft
	local filtered = (icons.whitelist~=nil) and icons.whitelist[name]
	return name and filtered
end

local function tableSize(table)
	local i = 0
	for index,_ in pairs(table)do
		i = i + 1
	end
	return i
end
--END ICON FRAME

local function updatePower(self, event, unit, bar, min, max)
	if(UnitPowerType(unit) == 0 and min<max) then
		self.Health:SetHeight(17)
		bar:Show()
	else
		self.Health:SetHeight(20)
		bar:Hide()
	end
end

local function CreateStyle(self, unit)

	self.DebuffHighlightBackdrop = true
	self.DebuffHighlightAlpha = 1
	
	self.menu = menu
	self.colors = colors
	self:RegisterForClicks('AnyUp')
	self:SetScript('OnEnter', UnitFrame_OnEnter)
	self:SetScript('OnLeave', UnitFrame_OnLeave)
	self:SetAttribute('*type2', 'menu')
	self:SetAttribute('initial-height', 20)
	self:SetAttribute('initial-width', 120)
	
	self:SetBackdrop({bgFile = [=[Interface\ChatFrame\ChatFrameBackground]=], insets = {top = -2, left = -2, bottom = -2, right = -2}})
	self:SetBackdropColor(0.25, 0.25, 0.25)

	self.Health = CreateFrame('StatusBar', nil, self)
	self.Health:SetPoint('TOPLEFT', self,"TOPLEFT")

	self.Health:SetPoint('TOPRIGHT', self,"TOPRIGHT")
	self.Health:SetHeight(20)
	self.Health:SetStatusBarTexture([=[Interface\AddOns\oUF_Perfect2\minimalist]=])
	self.Health.colorDisconnected = true
	self.Health.colorClass = true

	self.Health.bg = self.Health:CreateTexture(nil, 'BORDER')
	self.Health.bg:SetAllPoints(self.Health)
	self.Health.bg:SetTexture([=[Interface\ChatFrame\ChatFrameBackground]=])
	self.Health.bg.multiplier = 0.3

	local health = self.Health:CreateFontString(nil, 'OVERLAY', 'GameFontHighlightSmallRight')
	health:SetPoint('RIGHT', -3, 0)
	self:Tag(health, '[smarthp]')

	local power = self.Health:CreateFontString(nil, 'OVERLAY', 'GameFontHighlightSmallLeft')
	power:SetPoint('LEFT', 3, 0)
	self:Tag(power, '[smartpp]')
-- Mana Bar	
	self.Power = CreateFrame('StatusBar', nil, self)
	self.Power:SetPoint('TOPRIGHT', self.Health,"BOTTOMRIGHT")

	self.Power:SetPoint('TOPLEFT', self.Health,"BOTTOMLEFT")
	self.Power:SetStatusBarTexture([=[Interface\AddOns\oUF_Perfect2\minimalist]=])
	self.Power:SetHeight(3)

	self.Power.bg = self.Power:CreateTexture(nil, 'BORDER')
	self.Power.bg:SetAllPoints(self.Power)
	self.Power.bg:SetTexture([=[Interface\ChatFrame\ChatFrameBackground]=])

	self.Power.colorDisconnected = true
	self.Power.colorPower = true
	self.PostUpdatePower = updatePower

	
	self.RaidIcon = self.Health:CreateTexture(nil, 'OVERLAY')
	self.RaidIcon:SetPoint('TOP', self, 0, 8)
	self.RaidIcon:SetHeight(16)
	self.RaidIcon:SetWidth(16)
	
-- changed name font object template to a smaller one
	local name = self.Health:CreateFontString(nil, 'OVERLAY', 'GameTooltipTextSmall')
	name:SetPoint('TOPLEFT', self, 'TOPRIGHT', 5, 2)
	self:Tag(name, '[name( )][leader( )][offline( )][afk( )]|cff00ffff[(- )assistname( )][assisticon]')	
	--local name = self.Health:CreateFontString(nil, 'OVERLAY', 'GameFontHighlightLeft')
	--name:SetPoint('LEFT', self, 'RIGHT', 5, 0)
	--self:Tag(name, '[name( )][leader( )][offline( )][afk( )]|cff00ffff[(- )assistname( )][assisticon]')

	self.ReadyCheck = self.Health:CreateTexture(nil, 'OVERLAY')
	self.ReadyCheck:SetPoint('LEFT', self, -18, 0)
	self.ReadyCheck:SetHeight(20)
	self.ReadyCheck:SetWidth(20)

	self.MasterLooter = self.Health:CreateTexture(nil, 'OVERLAY')
	self.MasterLooter:SetPoint('TOPLEFT', self, 5, 0)
	self.MasterLooter:SetHeight(12)
	self.MasterLooter:SetWidth(12)
	
-- BEGIN BUFF FRAME
--------------------------
	self.Buffs = CreateFrame('Frame', nil, self)
--points to the table at the top
	self.Buffs.whitelist = auraWhiteList
--position it from the BottomLeft corner of the buffFrame to the BottomRight corner of the unitframe(self), 
-- offset it to the right by 5pixels and down by 1 pixel
	self.Buffs:SetPoint('BOTTOMLEFT', self, 'BOTTOMRIGHT', 5, -1)
-- set the number of buffs to display to be equal to the amount of entries in out whitelist.
	self.Buffs.num = tableSize(self.Buffs.whitelist)
-- the width and height of our buff icons
	self.Buffs.size = 13
-- spacing between the icons
	self.Buffs.spacing = 2	
-- where our first icon is anchored to within the buffframe.
	self.Buffs.initialAnchor = 'BOTTOMLEFT'
-- set the height of our buffframe to the same hieght as our icons size
	self.Buffs:SetHeight(self.Buffs.size)
-- set the width of the buffframe -equal to- the icon size -plus- icon spacing -times- the amount of entries in our whitelist
	self.Buffs:SetWidth( (self.Buffs.size+self.Buffs.spacing) * self.Buffs.num )
-- new icon rows grow down. we wont have any here because our row length always equals our whitelist size.
	self.Buffs['growth-y'] = 'DOWN'
-- new icons appear to the right of old ones.
	self.Buffs['growth-x'] = 'RIGHT'

-- this hooks the whitelist filter
	self.CustomAuraFilter = customFilter
-- this hooks tooltip text and buff canceling of your own buffs on your own frames. *not others*
	self.PostCreateAuraIcon = createAura


--END BUFF FRAME
-----------------------
	self.DebuffHighlightBackdrop = true
	self.DebuffHighlightFilter = true
	self.outsideRangeAlpha = 0.4
	self.inRangeAlpha = 1.0
	self.Range = true
end

oUF:RegisterStyle('Perfect2', CreateStyle)
oUF:SetActiveStyle('Perfect2')

--[[
List of the various configuration attributes
======================================================
showRaid = [BOOLEAN] -- true if the header should be shown while in a raid
showParty = [BOOLEAN] -- true if the header should be shown while in a party and not in a raid
showPlayer = [BOOLEAN] -- true if the header should show the player when not in a raid
showSolo = [BOOLEAN] -- true if the header should be shown while not in a group (implies showPlayer)
nameList = [STRING] -- a comma separated list of player names (not used if 'groupFilter' is set)
groupFilter = [1-8, STRING] -- a comma seperated list of raid group numbers and/or uppercase class names and/or uppercase roles
strictFiltering = [BOOLEAN] - if true, then characters must match both a group and a class from the groupFilter list
point = [STRING] -- a valid XML anchoring point (Default: "TOP")
xOffset = [NUMBER] -- the x-Offset to use when anchoring the unit buttons (Default: 0)
yOffset = [NUMBER] -- the y-Offset to use when anchoring the unit buttons (Default: 0)
sortMethod = ["INDEX", "NAME"] -- defines how the group is sorted (Default: "INDEX")
sortDir = ["ASC", "DESC"] -- defines the sort order (Default: "ASC")
template = [STRING] -- the XML template to use for the unit buttons
templateType = [STRING] - specifies the frame type of the managed subframes (Default: "Button")
groupBy = [nil, "GROUP", "CLASS", "ROLE"] - specifies a "grouping" type to apply before regular sorting (Default: nil)
groupingOrder = [STRING] - specifies the order of the groupings (ie. "1,2,3,4,5,6,7,8")
maxColumns = [NUMBER] - maximum number of columns the header will create (Default: 1)
unitsPerColumn = [NUMBER or nil] - maximum units that will be displayed in a singe column, nil is infinate (Default: nil)
startingIndex = [NUMBER] - the index in the final sorted unit list at which to start displaying units (Default: 1)
columnSpacing = [NUMBER] - the ammount of space between the rows/columns (Default: 0)
columnAnchorPoint = [STRING] - the anchor point of each new column (ie. use LEFT for the columns to grow to the right)
--]]

local raid = {}
for i = 1, 5 do
	local group = oUF:Spawn('header', 'oUF_Group'..i)
	group:SetManyAttributes('groupFilter', tostring(i), 'showRaid', true, 'yOffset', -5)
	group:SetFrameStrata('BACKGROUND')
	table.insert(raid, group)
	if(i == 1) then
		group:SetManyAttributes('showParty', true, 'showPlayer', true,'showSolo',true)
		group:SetPoint('TOPLEFT', UIParent, 15, -25)
	else
		group:SetPoint('TOP', raid[i-1], 'BOTTOM', 0, -25)
	end
	group:Show()
end


-- Debuff Highlight

local playerClass = select(2,UnitClass("player"))
local CanDispel = {
	PRIEST = { Magic = true, Disease = true, },
	SHAMAN = { Poison = true, Disease = true, Curse = true, },
	PALADIN = { Magic = true, Poison = true, Disease = true, },
	MAGE = { Curse = true, },
	DRUID = { Curse = true, Poison = true, }
}
local dispellist = CanDispel[playerClass] or {}
local origColors = {}
local origBorderColors = {}
local origPostUpdateAura = {}

local function GetDebuffType(unit, filter)
	if not UnitCanAssist("player", unit) then return nil end
	local i = 1
	while true do
		local _, _, texture, _, debufftype = UnitAura(unit, i, "HARMFUL")
		if not texture then break end
		if debufftype and not filter or (filter and dispellist[debufftype]) then
			return debufftype, texture
		end
		i = i + 1
	end
end

local function Update(object, event, unit)
	if object.unit ~= unit  then return end
	local debuffType, texture  = GetDebuffType(unit, object.DebuffHighlightFilter)
	if debuffType then
		local color = DebuffTypeColor[debuffType] 
		if object.DebuffHighlightBackdrop then
			object:SetBackdropColor(color.r, color.g, color.b, object.DebuffHighlightAlpha or 1)
		elseif object.DebuffHighlightUseTexture then
			object.DebuffHighlight:SetTexture(texture)
		else
			object.DebuffHighlight:SetVertexColor(color.r, color.g, color.b, object.DebuffHighlightAlpha or .5)
		end
	else
		if object.DebuffHighlightBackdrop then
			local color = origColors[object]
			object:SetBackdropColor(color.r, color.g, color.b, color.a)
			color = origBorderColors[object]
			object:SetBackdropBorderColor(color.r, color.g, color.b, color.a)
		elseif object.DebuffHighlightUseTexture then
			object.DebuffHighlight:SetTexture(nil)
		else
			local color = origColors[object]
			object.DebuffHighlight:SetVertexColor(color.r, color.g, color.b, color.a)
		end
	end
end

local function Enable(object)
	-- if we're not highlighting this unit return
	if not object.DebuffHighlightBackdrop and not object.DebuffHighlight then
		return
	end
	-- if we're filtering highlights and we're not of the dispelling type, return
	if object.DebuffHighlightFilter and not CanDispel[playerClass] then
		return
	end
	
	-- make sure aura scanning is active for this object
	object:RegisterEvent("UNIT_AURA", Update)
	
	if object.DebuffHighlightBackdrop then
		local r, g, b, a = object:GetBackdropColor()
		origColors[object] = { r = r, g = g, b = b, a = a}
		r, g, b, a = object:GetBackdropBorderColor()
		origBorderColors[object] = { r = r, g = g, b = b, a = a}
	elseif not object.DebuffHighlightUseTexture then -- color debuffs
		-- object.DebuffHighlight
		local r, g, b, a = object.DebuffHighlight:GetVertexColor()
		origColors[object] = { r = r, g = g, b = b, a = a}
	end

	return true
end

local function Disable(object)
	if object.DebuffHighlightBackdrop or object.DebuffHighlight then
		object:UnregisterEvent("UNIT_AURA", Update)
	end
end

oUF:AddElement('DebuffHighlight', Update, Enable, Disable)

for i, frame in ipairs(oUF.objects) do Enable(frame) end